@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -50,8 +50,6 @@ class ToUsenet:
         error = []
         if not mlist.linked_newsgroup:
             error.append('no newsgroup')
-        if not mlist.nntp_host:
-            error.append('no NNTP host')
         if error:
             log.error('NNTP gateway improperly configured: %s',
                       COMMASPACE.join(error))
