@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2021 by the Free Software Foundation, Inc.
+# Copyright (C) 2010-2023 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -17,31 +17,60 @@
 
 """REST for members."""
 
+import logging
+
 from lazr.config import as_boolean
 from mailman.app.membership import add_member
 from mailman.interfaces.action import Action
 from mailman.interfaces.address import IAddress, InvalidEmailAddressError
 from mailman.interfaces.listmanager import IListManager
 from mailman.interfaces.member import (
-    AlreadySubscribedError, DeliveryMode, MemberRole, MembershipError,
-    MembershipIsBannedError, MissingPreferredAddressError)
+    AlreadySubscribedError,
+    DeliveryMode,
+    DeliveryStatus,
+    MemberRole,
+    MembershipError,
+    MembershipIsBannedError,
+    MissingPreferredAddressError,
+)
 from mailman.interfaces.subscriptions import (
-    ISubscriptionManager, ISubscriptionService, RequestRecord,
-    SubscriptionPendingError, TokenOwner)
+    ISubscriptionManager,
+    ISubscriptionService,
+    RequestRecord,
+    SubscriptionPendingError,
+    TokenOwner,
+)
 from mailman.interfaces.user import IUser, UnverifiedAddressError
 from mailman.interfaces.usermanager import IUserManager
 from mailman.rest.helpers import (
-    CollectionMixin, NotFound, accepted, bad_request, child, conflict,
-    created, etag, no_content, not_found, okay)
+    accepted,
+    bad_request,
+    child,
+    CollectionMixin,
+    conflict,
+    created,
+    etag,
+    no_content,
+    not_found,
+    NotFound,
+    okay,
+)
 from mailman.rest.preferences import Preferences, ReadOnlyPreferences
 from mailman.rest.validator import (
-    Validator, enum_validator, list_of_strings_validator, subscriber_validator)
+    enum_validator,
+    list_of_strings_validator,
+    subscriber_validator,
+    Validator,
+)
 from operator import attrgetter
 from public import public
 from uuid import UUID
 from zope.component import getUtility
 
 
+log = logging.getLogger('mailman.rest')
+
+
 class _MemberBase(CollectionMixin):
     """Shared base class for member representations."""
 
@@ -61,6 +90,10 @@ class _MemberBase(CollectionMixin):
         # issue #121 for details.
         return {
             'address': self._get_address,
+            'bounce_score': attrgetter('bounce_score'),
+            'last_bounce_received': attrgetter('last_bounce_received'),
+            'last_warning_sent': attrgetter('last_warning_sent'),
+            'total_warnings_sent': attrgetter('total_warnings_sent'),
             'delivery_mode': attrgetter('delivery_mode'),
             'email': attrgetter('address.email'),
             'list_id': attrgetter('list_id'),
@@ -288,6 +321,7 @@ class AllMembers(_MemberBase):
                 subscriber=subscriber_validator(self.api),
                 display_name=str,
                 delivery_mode=enum_validator(DeliveryMode),
+                delivery_status=enum_validator(DeliveryStatus),
                 role=enum_validator(MemberRole),
                 pre_verified=as_boolean,
                 pre_confirmed=as_boolean,
@@ -296,7 +330,8 @@ class AllMembers(_MemberBase):
                 send_welcome_message=as_boolean,
                 _optional=('delivery_mode', 'display_name', 'role',
                            'pre_verified', 'pre_confirmed', 'pre_approved',
-                           'invitation', 'send_welcome_message',))
+                           'invitation', 'send_welcome_message',
+                           'delivery_status'))
             arguments = validator(request)
         except ValueError as error:
             bad_request(response, str(error))
@@ -340,6 +375,8 @@ class AllMembers(_MemberBase):
             pre_approved = arguments.pop('pre_approved', False)
             invitation = arguments.pop('invitation', False)
             send_welcome_message = arguments.pop('send_welcome_message', None)
+            delivery_status = arguments.pop('delivery_status', None)
+            delivery_mode = arguments.pop('delivery_mode', None)
             # Now we can run the registration process until either the
             # subscriber is subscribed, or the workflow is paused for
             # verification, confirmation, or approval.
@@ -351,7 +388,9 @@ class AllMembers(_MemberBase):
                     pre_confirmed=pre_confirmed,
                     pre_approved=pre_approved,
                     invitation=invitation,
-                    send_welcome_message=send_welcome_message)
+                    send_welcome_message=send_welcome_message,
+                    delivery_mode=delivery_mode,
+                    delivery_status=delivery_status)
             except AlreadySubscribedError:
                 conflict(response, b'Member already subscribed')
                 return
@@ -368,6 +407,7 @@ class AllMembers(_MemberBase):
                 conflict(response, b'Subscription request already pending')
                 return
             except Exception as e:
+                log.exception(e)
                 bad_request(response, str(e))
                 return
             if token is None:
@@ -413,7 +453,10 @@ class AllMembers(_MemberBase):
             assert IAddress.providedBy(subscriber)
             email = subscriber.email
         delivery_mode = arguments.pop('delivery_mode', DeliveryMode.regular)
-        record = RequestRecord(email, display_name, delivery_mode)
+        delivery_status = arguments.pop('delivery_status', None)
+        record = RequestRecord(
+            email, display_name, delivery_mode,
+            delivery_status=delivery_status)
         try:
             member = add_member(mlist, record, role)
         except MembershipIsBannedError:
@@ -466,12 +509,18 @@ class FindMembers(_MemberBase):
             list_id=str,
             subscriber=str,
             role=enum_validator(MemberRole),
+            moderation_action=enum_validator(Action, allow_blank=True),
+            # preferences.
+            delivery_mode=enum_validator(DeliveryMode),
+            delivery_status=enum_validator(DeliveryStatus),
             # Allow pagination.
             page=int,
             count=int,
             fields=list_of_strings_validator,
             _optional=(
-                'list_id', 'subscriber', 'role', 'page', 'count', 'fields'))
+                'list_id', 'subscriber', 'role', 'moderation_action',
+                'delivery_mode', 'delivery_status',
+                'page', 'count', 'fields'))
         try:
             data = validator(request)
         except ValueError as error:
