@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2011-2023 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -112,9 +112,9 @@ class TestDomains(unittest.TestCase):
         self.assertEqual(response.status_code, 204)
         # Check the result.
         domain = getUtility(IDomainManager).get('example.com')
-        self.assertEqual(
-            [list(owner.addresses)[0].email for owner in domain.owners],
-            ['anne@example.com', 'other@example.net'])
+        owners = [list(owner.addresses)[0].email for owner in domain.owners]
+        owners.sort()
+        self.assertEqual(owners, ['anne@example.com', 'other@example.net'])
 
     def test_patch_domain_readonly(self):
         # Attempt to patch mail_host.
@@ -374,12 +374,17 @@ class TestDomainTemplates(unittest.TestCase):
                 'list:admin:action:unsubscribe': '',
                 'list:admin:notice:subscribe': '',
                 'list:admin:notice:unrecognized': '',
+                'list:admin:notice:pending': '',
+                'list:admin:notice:increment': '',
+                'list:admin:notice:disable': '',
                 'list:admin:notice:unsubscribe': '',
+                'list:admin:notice:removal': '',
                 'list:member:digest:footer': '',
                 'list:member:digest:header': '',
                 'list:member:digest:masthead': '',
                 'list:member:regular:footer': 'http://example.org/footer',
                 'list:member:regular:header': 'http://example.org/header',
+                'list:user:action:invite': '',
                 'list:user:action:subscribe': '',
                 'list:user:action:unsubscribe': '',
                 'list:user:notice:goodbye': 'http://example.org/goodbye',
@@ -390,6 +395,7 @@ class TestDomainTemplates(unittest.TestCase):
                 'list:user:notice:refuse': '',
                 'list:user:notice:rejected': '',
                 'list:user:notice:welcome': 'http://example.org/welcome',
+                'list:user:notice:warning': '',
                 'password': 'some password',
                 'username': 'anne.person',
                 }, method='PUT')
