@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2001-2023 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -19,10 +19,14 @@
 
 import logging
 
-from flufl.bounce import all_failures, scan_message
-from mailman.app.bounces import ProbeVERP, StandardVERP, maybe_forward
+from flufl.bounce import all_failures
+from mailman.app.bounces import maybe_forward, ProbeVERP, StandardVERP
 from mailman.core.runner import Runner
-from mailman.interfaces.bounce import BounceContext, IBounceProcessor
+from mailman.interfaces.bounce import (
+    BounceContext,
+    IBounceProcessor,
+    InvalidBounceEvent,
+)
 from public import public
 from zope.component import getUtility
 
@@ -50,13 +54,18 @@ class BounceRunner(Runner):
         addresses = StandardVERP().get_verp(mlist, msg)
         if len(addresses) > 0:
             # Scan the message to see if it contained permanent or temporary
-            # failures.  We'll ignore temporary failures, but even if there
-            # are no permanent failures, we'll assume VERP bounces are
-            # permanent.
+            # failures.  We'll ignore temporary failures, and if there
+            # are no permanent failures, we'll assume this is a vacation
+            # response or similar.
             temporary, permanent = all_failures(msg)
             if len(temporary) > 0:
                 # This was a temporary failure, so just ignore it.
                 return False
+            if len(permanent) == 0:
+                log.info('VERPed bounce message but not a recognized DSN: %s',
+                         msg.get('message-id', 'n/a'))
+                maybe_forward(mlist, msg)
+                return False
         else:
             # See if this was a probe message.
             addresses = ProbeVERP().get_verp(mlist, msg)
@@ -64,11 +73,14 @@ class BounceRunner(Runner):
                 context = BounceContext.probe
             else:
                 # That didn't give us anything useful, so try the old fashion
-                # bounce matching modules.  This returns only the permanently
-                # failing addresses.  Since Mailman currently doesn't score
-                # temporary failures, if we get no permanent failures, we're
-                # done.s
-                addresses = scan_message(msg)
+                # bounce matching modules.  Since Mailman currently doesn't
+                # score temporary failures, if we get no permanent failures,
+                # we're done, but we do need to check for temporary failures
+                # to know if the bounce was recognized.
+                temporary, addresses = all_failures(msg)
+                if len(addresses) == 0 and len(temporary) > 0:
+                    # This is a recognized temp fail so ignore it.
+                    return False
         # If that still didn't return us any useful addresses, then send it on
         # or discard it.  The addresses will come back from flufl.bounce as
         # bytes/8-bit strings, but we must store them as unicodes in the
@@ -89,3 +101,24 @@ class BounceRunner(Runner):
             maybe_forward(mlist, msg)
         # Dequeue this message.
         return False
+
+    def _do_periodic(self):
+        """Invoked periodically by the run() method in the super class."""
+        self._process_events()
+        self._send_warnings()
+
+    def _process_events(self):
+        """Process all the pending bounce events."""
+        log.debug('Processing bounce events.')
+        for bounce_event in self._processor.unprocessed:
+            try:
+                self._processor.process_event(bounce_event)
+            except InvalidBounceEvent as e:
+                # This member is either unsubscribed or this is a very stale
+                # event.
+                log.info('Bounce message for a non subscriber: {}'.format(e))
+
+    def _send_warnings(self):
+        """Send warnings to disabled users and remove them if needed."""
+        log.debug('Sending warnings to members with disabled delivery.')
+        self._processor.send_warnings_and_remove()
