@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2012-2023 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -31,7 +31,12 @@ from mailman.interfaces.autorespond import ResponseAction
 from mailman.interfaces.bounce import UnrecognizedBounceDisposition
 from mailman.interfaces.digests import DigestFrequency
 from mailman.interfaces.mailinglist import (
-    DMARCMitigateAction, Personalization, ReplyToMunging, SubscriptionPolicy)
+    ArchiveRenderingMode,
+    DMARCMitigateAction,
+    Personalization,
+    ReplyToMunging,
+    SubscriptionPolicy,
+)
 from mailman.interfaces.nntp import NewsgroupModeration
 from mailman.model.roster import RosterVisibility
 from public import public
@@ -51,7 +56,7 @@ class Identity:
         mlist.description = ''
         mlist.info = ''
         mlist.preferred_language = 'en'
-        mlist.subject_prefix = _('[$mlist.display_name] ')
+        mlist.subject_prefix = _('[${mlist.display_name}] ')
         mlist.encode_ascii_prefixes = (
             mlist.preferred_language.charset != 'us-ascii')
 
@@ -92,7 +97,6 @@ class BasicOperation:
         mlist.dmarc_moderation_notice = ''
         mlist.dmarc_wrapped_message_text = ''
         # NNTP gateway
-        mlist.nntp_host = ''
         mlist.linked_newsgroup = ''
         mlist.gateway_to_news = False
         mlist.gateway_to_mail = False
@@ -133,6 +137,8 @@ class BasicOperation:
         mlist.topics_userinterest = {}
         # scrub regular delivery
         mlist.scrub_nondigest = False
+        # Archive rendering mode.
+        mlist.archive_rendering_mode = ArchiveRenderingMode.text
 
 
 @public
@@ -150,6 +156,7 @@ class Bounces:
         mlist.bounce_info_stale_after = timedelta(days=7)
         mlist.bounce_you_are_disabled_warnings = 3
         mlist.bounce_you_are_disabled_warnings_interval = timedelta(days=7)
+        mlist.bounce_notify_owner_on_bounce_increment = False
         mlist.bounce_notify_owner_on_disable = True
         mlist.bounce_notify_owner_on_removal = True
         # Autoresponder
