@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2010-2023 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -26,6 +26,8 @@ import datetime
 
 from contextlib import ExitStack
 from mailman.config import config
+from mailman.database.helpers import is_mysql
+from mailman.database.types import SAUnicode, SAUnicode4Byte
 from mailman.handlers.decorate import decorate_template
 from mailman.interfaces.action import Action, FilterAction
 from mailman.interfaces.address import IEmailValidator
@@ -37,12 +39,18 @@ from mailman.interfaces.digests import DigestFrequency
 from mailman.interfaces.errors import MailmanError
 from mailman.interfaces.languages import ILanguageManager
 from mailman.interfaces.mailinglist import (
-    DMARCMitigateAction, IAcceptableAliasSet, IHeaderMatchList,
-    Personalization, ReplyToMunging, SubscriptionPolicy)
+    DMARCMitigateAction,
+    IAcceptableAliasSet,
+    IHeaderMatchList,
+    Personalization,
+    ReplyToMunging,
+    SubscriptionPolicy,
+)
 from mailman.interfaces.member import DeliveryMode, DeliveryStatus, MemberRole
 from mailman.interfaces.nntp import NewsgroupModeration
-from mailman.interfaces.template import ITemplateLoader, ITemplateManager
+from mailman.interfaces.template import ITemplateLoader
 from mailman.interfaces.usermanager import IUserManager
+from mailman.model.roster import RosterVisibility
 from mailman.utilities.filesystem import makedirs
 from mailman.utilities.i18n import search
 from public import public
@@ -50,6 +58,7 @@ from sqlalchemy import Boolean
 from urllib.error import URLError
 from zope.component import getUtility
 
+
 log = logging.getLogger('mailman.error')
 
 
@@ -134,15 +143,27 @@ def member_moderation_action_mapping(value):
 def nonmember_action_mapping(value):
     # For default_nonmember_action, which used to be called
     # generic_nonmember_action, the values were: 0==Accept, 1==Hold,
-    # 2==Reject, 3==Discard
+    # 2==Reject, 3==Discard, but note that Accept is really equivalent to
+    # what is now defer.
     return {
-        0: Action.accept,
+        0: Action.defer,
         1: Action.hold,
         2: Action.reject,
         3: Action.discard,
         }[value]
 
 
+def member_roster_visibility_mapping(value):
+    # For member_roster_visibility, which used to be called private_roster.
+    # The values were: 0==public, 1==members, 2==admins.
+    mapping = {
+        0: RosterVisibility.public,
+        1: RosterVisibility.members,
+        2: RosterVisibility.moderators,
+        }
+    return mapping.get(value, None)
+
+
 def action_to_chain(value):
     # Converts an action number in Mailman 2.1 to the name of the corresponding
     # chain in 3.x.  The actions 'approve', 'subscribe' and 'unsubscribe' are
@@ -180,6 +201,19 @@ enabled: yes
     return code
 
 
+def maybe_truncate_mysql(value):
+    # For MySQL, column type SAUnicode is VARCHAR(255).  In many MySQL
+    # configurations, attempts to store longer values are fatal.
+    if is_mysql(config.db.engine) and len(value) > 255:
+        # This actually is covered but only if db is MySQL.
+        print(                                # pragma: nocover
+              """Length of value for {} is {} which is too long for MySQL.
+Truncated from {}
+to {}""".format(key, len(value), value, value[:255]), file=sys.stderr)
+        return value[:255]                    # pragma: nocover
+    return value
+
+
 # Attributes in Mailman 2 which have a different type in Mailman 3.  Some
 # types (e.g. bools) are autodetected from their SA column types.
 TYPES = dict(
@@ -237,9 +271,13 @@ DATETIME_COLUMNS = [
     ]
 
 EXCLUDES = set((
+    'accept_these_nonmembers',
     'delivery_status',
     'digest_members',
+    'discard_these_nonmembers',
+    'hold_these_nonmembers',
     'members',
+    'reject_these_nonmembers',
     'user_options',
     ))
 
@@ -253,6 +291,7 @@ def import_config_pck(mlist, config_dict):
     :param config_dict: The Mailman 2.1 configuration dictionary.
     :type config_dict: dict
     """
+    global key
     for key, value in config_dict.items():
         # Some attributes must not be directly imported.
         if key in EXCLUDES:
@@ -278,6 +317,10 @@ def import_config_pck(mlist, config_dict):
                 column = getattr(mlist.__class__, key, None)
                 if column is not None and isinstance(column.type, Boolean):
                     converter = bool
+                if column is not None \
+                        and (isinstance(column.type, SAUnicode)
+                             or isinstance(column.type, SAUnicode4Byte)):
+                    converter = maybe_truncate_mysql
             try:
                 if converter is not None:
                     value = converter(value)
@@ -367,6 +410,13 @@ def import_config_pck(mlist, config_dict):
         # but .add() would not raise ValueError if address contained '@' and
         # that needs the '^' too as it could be a regexp with an '@' in it.
         alias_set.add(address)
+    # MM 2.1 accepts the listname as a local part regardless of domain.
+    alias_set.add('^' + mlist.list_name + '@')
+    # Handle roster visibility.
+    mapping = member_roster_visibility_mapping(
+        config_dict.get('private_roster', None))
+    if mapping is not None:
+        mlist.member_roster_visibility = mapping
     # Handle header_filter_rules conversion to header_matches.
     header_matches = IHeaderMatchList(mlist)
     header_filter_rules = config_dict.get('header_filter_rules', [])
@@ -420,7 +470,7 @@ def import_config_pck(mlist, config_dict):
     # special `mailman:` scheme indicating a file system path.  What we do
     # here is look to see if the list's decoration is different than the
     # default, and if so, we'll write the new decoration template to a
-    # `mailman:` scheme path, then add the template to the template manager.
+    # `mailman:` scheme path which is in the standard search path.
     # We are intentionally omitting the 2.1 welcome_msg here because the
     # string is actually interpolated into a larger template and there's
     # no good way to figure where in the default template to insert it.
@@ -435,15 +485,26 @@ def import_config_pck(mlist, config_dict):
     # order dependent; the longer substitution with the common prefix must
     # show up earlier.
     convert_placeholders = [
+        # First convert \r\n that may have been set by a browser to \n.
+        ('\r\n', '\n'),
         ('%(real_name)s@%(host_name)s',
          'To unsubscribe send an email to ${short_listname}-leave@${domain}'),
         ('%(real_name)s mailing list',
-         '$display_name mailing list -- $listname'),
+         '${display_name} mailing list -- ${listname}'),
         # The generic footers no longer have URLs in them.
         ('%(web_page_url)slistinfo%(cgiext)s/%(_internal_name)s\n', ''),
+        # Additional simple replacement conversions.
+        ('%(real_name)s', '${display_name}'),
+        ('%(list_name)s', '${listname}'),
+        ('%(description)s', '${description}'),
+        ('%(info)s', '${info}'),
+        ('%(cgiext)s', ''),
+        ('%(user_address)s', '${user_email}'),
+        ('%(user_delivered_to)s', '${user_delivered_to}'),
+        ('%(user_password)s', ''),
+        ('%(user_name)s', '${user_name}'),
         ]
     # Collect defaults.
-    manager = getUtility(ITemplateManager)
     defaults = {}
     for oldvar, newvar in convert_to_uri.items():
         default_value = getUtility(ITemplateLoader).get(newvar, mlist)
@@ -458,7 +519,7 @@ def import_config_pck(mlist, config_dict):
             # more harm than good and it's easy to change if needed.
             # TESTME
             print('Unable to convert mailing list attribute:', oldvar,
-                  'with old value "{}"'.format(default_value),
+                  'with bad replacements in "{}"'.format(default_value),
                   file=sys.stderr)
             continue
         defaults[newvar] = default_text
@@ -491,11 +552,12 @@ def import_config_pck(mlist, config_dict):
                 expanded_text.strip() == default_text.strip()):
             # Keep the default.
             continue
-        # Write the custom value to the right file and add it to the template
-        # manager for real.
-        base_uri = 'mailman:///$listname/$language/'
+        if expanded_text and '%' in expanded_text:
+            print('Unable to convert mailing list attribute:', oldvar,
+                  'with value "{}"'.format(text),
+                  file=sys.stderr)
+        # Write the custom value to the right file.
         filename = '{}.txt'.format(newvar)
-        manager.set(newvar, mlist.list_id, base_uri + filename)
         with ExitStack() as resources:
             filepath = list(search(resources, filename, mlist))[0]
         makedirs(os.path.dirname(filepath))
@@ -505,9 +567,11 @@ def import_config_pck(mlist, config_dict):
     regulars_set = set(config_dict.get('members', {}))
     digesters_set = set(config_dict.get('digest_members', {}))
     members = regulars_set.union(digesters_set)
-    # Don't send welcome messages when we import the rosters.
+    # Don't send welcome messages or notify admins when we import the rosters.
     send_welcome_message = mlist.send_welcome_message
     mlist.send_welcome_message = False
+    admin_notify_mchanges = mlist.admin_notify_mchanges
+    mlist.admin_notify_mchanges = False
     try:
         import_roster(mlist, config_dict, members, MemberRole.member)
         import_roster(mlist, config_dict, config_dict.get('owner', []),
@@ -521,14 +585,19 @@ def import_config_pck(mlist, config_dict):
             emails = [addr
                       for addr in config_dict.get(prop_name, [])
                       if not addr.startswith('^')]
+            # MM 2.1 accept maps to MM 3 defer
+            if action_name == 'accept':
+                action_name = 'defer'
             import_roster(mlist, config_dict, emails, MemberRole.nonmember,
                           Action[action_name])
-            # Only keep the regexes in the legacy list property.
+            # Now add the regexes in the legacy list property.
             list_prop = getattr(mlist, prop_name)
-            for email in emails:
-                list_prop.remove(email)
+            for addr in config_dict.get(prop_name, []):
+                if addr.startswith('^'):
+                    list_prop.append(addr)
     finally:
         mlist.send_welcome_message = send_welcome_message
+        mlist.admin_notify_mchanges = admin_notify_mchanges
 
 
 def import_roster(mlist, config_dict, members, role, action=None):
@@ -550,8 +619,7 @@ def import_roster(mlist, config_dict, members, role, action=None):
     """
     name = (action and action.name) or role.name
     with click.progressbar(
-            members, label='Importing {} {:<10}'.format(
-                mlist.list_id, name + 's')) as iterator:
+            members, label='Importing {:<10}'.format(name + 's')) as iterator:
         _import_roster(mlist, config_dict, iterator, role, action=action)
 
 
@@ -565,10 +633,16 @@ def _import_roster(mlist, config_dict, members, role, action=None):
     validator = getUtility(IEmailValidator)
     roster = mlist.get_roster(role)
     skipped = []
+    banned = []
+    action_arg = action
     for email in members:
         # For owners and members, the emails can have a mixed case, so
         # lowercase them all.
         email = bytes_to_str(email).lower()
+        # Ignore any banned addresses for all rosters.
+        if IBanManager(mlist).is_banned(email):
+            banned.append((email, role))
+            continue
         if roster.get_member(email) is not None:
             skipped.append((email, role))
             continue
@@ -616,21 +690,33 @@ def _import_roster(mlist, config_dict, members, role, action=None):
                 bytes_to_str(config_dict['usernames'][email])
             user.display_name = \
                 bytes_to_str(config_dict['usernames'][email])
-        if email in config_dict.get('passwords', {}):
-            user.password = config.password_context.encrypt(
-                config_dict['passwords'][email])
-        # delivery_status
-        oldds = config_dict.get('delivery_status', {}).get(email, (0, 0))[0]
-        if oldds == 0:
+
+        # XXX(abraj): Importing passwords are very slow since encryption takes
+        # a lot of time and tends to slow down the import process a lot,
+        # especially for large lists. There is really no use of user passwords
+        # from list config. For now, this code is just being commented out in
+        # case some use case pops up.
+        #
+        # if email in config_dict.get('passwords', {}):
+        #     user.password = config.password_context.encrypt(
+        #         config_dict['passwords'][email])
+        # delivery_status - for members. owners/moderators are always enabled.
+        if role in (MemberRole.owner, MemberRole.moderator):
             member.preferences.delivery_status = DeliveryStatus.enabled
-        elif oldds == 1:
-            member.preferences.delivery_status = DeliveryStatus.unknown
-        elif oldds == 2:
-            member.preferences.delivery_status = DeliveryStatus.by_user
-        elif oldds == 3:
-            member.preferences.delivery_status = DeliveryStatus.by_moderator
-        elif oldds == 4:
-            member.preferences.delivery_status = DeliveryStatus.by_bounces
+        else:
+            oldds = config_dict.get(
+                'delivery_status', {}).get(email, (0, 0))[0]
+            if oldds == 0:
+                member.preferences.delivery_status = DeliveryStatus.enabled
+            elif oldds == 1:
+                member.preferences.delivery_status = DeliveryStatus.unknown
+            elif oldds == 2:
+                member.preferences.delivery_status = DeliveryStatus.by_user
+            elif oldds == 3:
+                member.preferences.delivery_status = \
+                    DeliveryStatus.by_moderator
+            elif oldds == 4:
+                member.preferences.delivery_status = DeliveryStatus.by_bounces
         # Moderation.
         if prefs is not None:
             # We're adding a member.
@@ -647,6 +733,10 @@ def _import_roster(mlist, config_dict, members, role, action=None):
             # Either this was set right above or in the function's arguments
             # for nonmembers.
             member.moderation_action = action
+            # We need to restore the action argument in case we changed it
+            # above so the changed action is not applied to the remaining
+            # members.
+            action = action_arg
         # Other preferences.
         if prefs is not None:
             # AcknowledgePosts
@@ -660,3 +750,6 @@ def _import_roster(mlist, config_dict, members, role, action=None):
     for email, role in skipped:
         print('{} is already imported with role {}'.format(email, role),
               file=sys.stderr)
+    for email, role in banned:
+        print('{} is banned and not imported with role {}'.format(email, role),
+              file=sys.stderr)
