@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2010-2023 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -20,11 +20,14 @@
 import os
 import unittest
 
-from datetime import timedelta, datetime
+from contextlib import ExitStack, redirect_stderr
+from datetime import datetime, timedelta
 from enum import Enum
 from importlib_resources import open_binary
+from io import StringIO
 from mailman.app.lifecycle import create_list
 from mailman.config import config
+from mailman.database.helpers import is_mysql
 from mailman.handlers.decorate import decorate
 from mailman.interfaces.action import Action, FilterAction
 from mailman.interfaces.address import InvalidEmailAddressError
@@ -35,17 +38,26 @@ from mailman.interfaces.bounce import UnrecognizedBounceDisposition
 from mailman.interfaces.domain import IDomainManager
 from mailman.interfaces.languages import ILanguageManager
 from mailman.interfaces.mailinglist import (
-    DMARCMitigateAction, IAcceptableAliasSet, SubscriptionPolicy)
+    DMARCMitigateAction,
+    IAcceptableAliasSet,
+    SubscriptionPolicy,
+)
 from mailman.interfaces.member import DeliveryMode, DeliveryStatus
 from mailman.interfaces.nntp import NewsgroupModeration
 from mailman.interfaces.template import ITemplateLoader, ITemplateManager
 from mailman.interfaces.usermanager import IUserManager
+from mailman.model.roster import RosterVisibility
 from mailman.testing.helpers import LogFileMark
 from mailman.testing.layers import ConfigLayer
 from mailman.utilities.filesystem import makedirs
+from mailman.utilities.i18n import search
 from mailman.utilities.importer import (
-    Import21Error, check_language_code, import_config_pck)
+    check_language_code,
+    Import21Error,
+    import_config_pck,
+)
 from pickle import load
+from shutil import rmtree
 from unittest import mock
 from urllib.error import URLError
 from zope.component import getUtility
@@ -320,8 +332,8 @@ class TestBasicImport(unittest.TestCase):
 
     def test_ban_list(self):
         banned = [
-            ('anne@example.com', 'anne@example.com'),
-            ('^.*@example.com', 'bob@example.com'),
+            ('anne@example.net', 'anne@example.net'),
+            ('^.*@example.edu', 'bob@example.edu'),
             ('non-ascii-\xe8@example.com', 'non-ascii-\ufffd@example.com'),
             ]
         self._pckdict['ban_list'] = [b[0].encode('iso-8859-1') for b in banned]
@@ -333,45 +345,54 @@ class TestBasicImport(unittest.TestCase):
         # This used to be a plain-text field (values are newline-separated)
         # but values were interpreted as regexps even without '^' so we need
         # to add the '^'.
+        # Also check that the list name local part is added.
         aliases = ['alias1@example.com',
                    'alias2@exemple.com',
                    'non-ascii-\xe8@example.com',
                    ]
+        new_aliases = ['^alias1@example.com',
+                       '^alias2@exemple.com',
+                       '^blank@',
+                       '^non-ascii-\xe8@example.com',
+                       ]
         self._pckdict['acceptable_aliases'] = list_to_string(aliases)
         self._import()
         alias_set = IAcceptableAliasSet(self._mlist)
-        self.assertEqual(sorted(alias_set.aliases),
-                         [('^' + alias) for alias in aliases])
+        self.assertEqual(sorted(alias_set.aliases), new_aliases)
 
     def test_acceptable_aliases_invalid(self):
         # Values without an '@' sign used to be matched against the local
         # part, now we need to add the '^' sign to indicate it's a regexp.
         aliases = ['invalid-value']
+        new_aliases = ['^blank@', '^invalid-value']
         self._pckdict['acceptable_aliases'] = list_to_string(aliases)
         self._import()
         alias_set = IAcceptableAliasSet(self._mlist)
-        self.assertEqual(sorted(alias_set.aliases),
-                         [('^' + alias) for alias in aliases])
+        self.assertEqual(sorted(alias_set.aliases), new_aliases)
 
     def test_acceptable_aliases_as_list(self):
         # In some versions of the pickle, this can be a list, not a string
         # (seen in the wild).  We still need to add the '^'.
         aliases = [b'alias1@example.com', b'alias2@exemple.com']
+        new_aliases = ['^alias1@example.com', '^alias2@exemple.com', '^blank@']
         self._pckdict['acceptable_aliases'] = aliases
         self._import()
         alias_set = IAcceptableAliasSet(self._mlist)
-        self.assertEqual(sorted(alias_set.aliases),
-                         sorted(('^' + a.decode('utf-8')) for a in aliases))
+        self.assertEqual(sorted(alias_set.aliases), new_aliases)
 
     def test_dont_add_caret_if_present(self):
         # The 2.1 alias could have had a leading '^' even though not required.
         aliases = ['^alias1@example.com',
                    '^alias2@.*',
                    ]
+        new_aliases = ['^alias1@example.com',
+                       '^alias2@.*',
+                       '^blank@',
+                       ]
         self._pckdict['acceptable_aliases'] = list_to_string(aliases)
         self._import()
         alias_set = IAcceptableAliasSet(self._mlist)
-        self.assertEqual(sorted(alias_set.aliases), aliases)
+        self.assertEqual(sorted(alias_set.aliases), new_aliases)
 
     def test_info_non_ascii(self):
         # info can contain non-ascii characters.
@@ -602,6 +623,22 @@ class TestBasicImport(unittest.TestCase):
         self.assertIn('Skipping duplicate header_filter rule',
                       error_log.readline())
 
+    def test_long_saunicode(self):
+        # Long SAUnicode fields should truncate for MySql with warning only.
+        long_desc = (
+            'A very long description exceeding 255 ckaracters to test '
+            'truncation of SAUnicode field data for MySQL. just add some '
+            'dots .......................................................'
+            '............................................................'
+            '... and a bit more Thats 255 ending at more')
+        self._pckdict['description'] = long_desc
+        self._import()
+        if is_mysql(config.db.engine):
+            self.assertEqual(long_desc[:255], self._mlist.description)
+            self.assertTrue(self._mlist.description.endswith('a bit more'))
+        else:
+            self.assertEqual(long_desc, self._mlist.description)
+
 
 class TestArchiveImport(unittest.TestCase):
     """Test conversion of the archive policies.
@@ -738,7 +775,7 @@ class TestMemberActionImport(unittest.TestCase):
 
     def test_nonmember_accept(self):
         self._pckdict['generic_nonmember_action'] = 0
-        self._do_test(dict(default_nonmember_action=Action.accept))
+        self._do_test(dict(default_nonmember_action=Action.defer))
 
     def test_nonmember_hold(self):
         self._pckdict['generic_nonmember_action'] = 1
@@ -784,6 +821,10 @@ class TestConvertToURI(unittest.TestCase):
             digest_footer='list:member:digest:footer',
             )
         self._pckdict = dict()
+        # Remove any residual template files.
+        with ExitStack() as resources:
+            filepath = list(search(resources, '', self._mlist))[0]
+        rmtree(filepath, ignore_errors=True)
 
     def test_text_to_uri(self):
         for oldvar, newvar in self._conf_mapping.items():
@@ -798,8 +839,8 @@ class TestConvertToURI(unittest.TestCase):
     def test_substitutions(self):
         test_text = ('UNIT TESTING %(real_name)s mailing list\n'
                      '%(real_name)s@%(host_name)s')
-        expected_text = ('UNIT TESTING $display_name mailing list '
-                         '-- $listname\n'
+        expected_text = ('UNIT TESTING ${display_name} mailing list '
+                         '-- ${listname}\n'
                          'To unsubscribe send an email to '
                          '${short_listname}-leave@${domain}')
         for oldvar, newvar in self._conf_mapping.items():
@@ -810,6 +851,101 @@ class TestConvertToURI(unittest.TestCase):
                 text, expected_text,
                 'Old variables were not converted for %s' % newvar)
 
+    def test_more_substitutions(self):
+        test_text = ('UNIT TESTING %(real_name)s some list\n'
+                     'The listname is %(list_name)s\n'
+                     'The description is %(description)s\n'
+                     'Info is %(info)s\n'
+                     'User address to is %(user_address)s\n'
+                     'User delivered is %(user_delivered_to)s\n'
+                     'User password: %(user_password)s\n'
+                     'User name %(user_name)s\n')
+        expected_text = ('UNIT TESTING ${display_name} some list\n'
+                         'The listname is ${listname}\n'
+                         'The description is ${description}\n'
+                         'Info is ${info}\n'
+                         'User address to is ${user_email}\n'
+                         'User delivered is ${user_delivered_to}\n'
+                         'User password: \n'
+                         'User name ${user_name}\n')
+        for oldvar, newvar in self._conf_mapping.items():
+            self._pckdict[str(oldvar)] = str(test_text)
+            import_config_pck(self._mlist, self._pckdict)
+            text = getUtility(ITemplateLoader).get(newvar, self._mlist)
+            self.assertEqual(
+                text, expected_text,
+                'Old variables were not converted for %s' % newvar)
+
+    def test_cgiext_dropped(self):
+        test_text = ('UNIT TESTING %(real_name)s mailing list\n'
+                     'https://example.com/mailman/listinfo%(cgiext)s')
+        expected_text = ('UNIT TESTING ${display_name} mailing list '
+                         '-- ${listname}\n'
+                         'https://example.com/mailman/listinfo')
+        for oldvar, newvar in self._conf_mapping.items():
+            self._pckdict[str(oldvar)] = str(test_text)
+            import_config_pck(self._mlist, self._pckdict)
+            text = getUtility(ITemplateLoader).get(newvar, self._mlist)
+            self.assertEqual(
+                text, expected_text,
+                'Old variables were not converted for %s' % newvar)
+
+    def test_unconverted_issues_message(self):
+        test_text = ('UNIT TESTING %(real_name)s mailing list\n'
+                     '%(web_page_url)slistinfo/')
+        expected_text = ('UNIT TESTING ${display_name} mailing list '
+                         '-- ${listname}\n'
+                         '%(web_page_url)slistinfo/')
+        for oldvar, newvar in self._conf_mapping.items():
+            serr = StringIO()
+            with redirect_stderr(serr):
+                self._pckdict[str(oldvar)] = str(test_text)
+                import_config_pck(self._mlist, self._pckdict)
+                text = getUtility(ITemplateLoader).get(newvar, self._mlist)
+                self.assertEqual(
+                    text, expected_text,
+                    'Old variables were not converted for %s' % newvar)
+            self.assertEqual(serr.getvalue(),
+                             'Unable to convert mailing list attribute: '
+                             f'{oldvar} with value "UNIT TESTING '
+                             '${display_name} mailing list -- ${listname}\n'
+                             '%(web_page_url)slistinfo/"\n')
+            serr.close()
+
+    def test_no_unconverted_issues_no_message(self):
+        test_text = ('UNIT TESTING %(real_name)s mailing list\n'
+                     'List description: %(description)s')
+        expected_text = ('UNIT TESTING ${display_name} mailing list '
+                         '-- ${listname}\n'
+                         'List description: ${description}')
+        for oldvar, newvar in self._conf_mapping.items():
+            serr = StringIO()
+            with redirect_stderr(serr):
+                self._pckdict[str(oldvar)] = str(test_text)
+                import_config_pck(self._mlist, self._pckdict)
+                text = getUtility(ITemplateLoader).get(newvar, self._mlist)
+                self.assertEqual(
+                    text, expected_text,
+                    'Old variables were not converted for %s' % newvar)
+            self.assertEqual(serr.getvalue(), '')
+            serr.close()
+
+    def test_drop_listinfo_if_contains_crs(self):
+        self._pckdict = dict(msg_footer=(
+            '_______________________________________________\r\n'
+            'This is a message footer\r\n'
+            '%(web_page_url)slistinfo%(cgiext)s/%(_internal_name)s\r\n'
+            ))
+        expected_text = (
+            '_______________________________________________\n'
+            'This is a message footer\n'
+            )
+        import_config_pck(self._mlist, self._pckdict)
+        newvar = 'list:member:regular:footer'
+        text = getUtility(ITemplateLoader).get(newvar, self._mlist)
+        self.assertEqual(
+            text, expected_text, 'Listinfo URL not dropped')
+
     def test_keep_default(self):
         # If the value was not changed from MM2.1's default, don't import it.
         default_msg_footer = (
@@ -836,6 +972,33 @@ class TestConvertToURI(unittest.TestCase):
                 '{} changed unexpectedly: {} != {}'.format(
                     newvar, old_value, new_value))
 
+    def test_keep_default_with_crs(self):
+        # If the value was not changed from MM2.1's default, don't import it.
+        default_msg_footer = (
+            '_______________________________________________\n'
+            '%(real_name)s mailing list\n'
+            '%(real_name)s@%(host_name)s\n'
+            '%(web_page_url)slistinfo%(cgiext)s/%(_internal_name)s\n'
+            )
+        loader = getUtility(ITemplateLoader)
+        for oldvar in ('msg_footer', 'digest_footer'):
+            newvar = self._conf_mapping[oldvar]
+            self._pckdict[str(oldvar)] = str(default_msg_footer).replace(
+                '\n', '\r\n')
+            try:
+                old_value = loader.get(newvar, self._mlist)
+            except URLError:
+                old_value = None
+            import_config_pck(self._mlist, self._pckdict)
+            try:
+                new_value = loader.get(newvar, self._mlist)
+            except URLError:
+                new_value = None
+            self.assertEqual(
+                old_value, new_value,
+                '{} changed unexpectedly: {} != {}'.format(
+                    newvar, old_value, new_value))
+
     def test_keep_default_if_fqdn_changed(self):
         # Use case: importing the old a@ex.com into b@ex.com.  We can't check
         # if it changed from the default so don't import.  We may do more harm
@@ -1042,16 +1205,17 @@ class TestRosterImport(unittest.TestCase):
                          [a.email for a in self._mlist.members.addresses],
                          'Address fred@ was wrongly added to the members list')
 
-    def test_password(self):
-        # self.anne.password = config.password_context.encrypt('abc123')
-        import_config_pck(self._mlist, self._pckdict)
-        for name in ('anne', 'bob', 'cindy', 'dave'):
-            addr = '%s@example.com' % name
-            user = self._usermanager.get_user(addr)
-            self.assertIsNotNone(user, 'Address %s was not imported' % addr)
-            self.assertEqual(
-                user.password, '{plaintext}%spass' % name,
-                'Password for %s was not imported' % addr)
+    # Commented out because password importing has been disabled.
+    # def test_password(self):
+    #     # self.anne.password = config.password_context.encrypt('abc123')
+    #     import_config_pck(self._mlist, self._pckdict)
+    #     for name in ('anne', 'bob', 'cindy', 'dave'):
+    #         addr = '%s@example.com' % name
+    #         user = self._usermanager.get_user(addr)
+    #         self.assertIsNotNone(user, 'Address %s was not imported' % addr)
+    #         self.assertEqual(
+    #             user.password, '{plaintext}%spass' % name,
+    #             'Password for %s was not imported' % addr)
 
     def test_same_user(self):
         # Adding the address of an existing User must not create another user.
@@ -1061,6 +1225,21 @@ class TestRosterImport(unittest.TestCase):
         member = self._mlist.members.get_member('bob@example.com')
         self.assertEqual(member.user, user)
 
+    def test_owner_and_moderator_delivery_enabled(self):
+        # If an owner or moderator is a member with delivery disabled, the
+        # imported owner/moderator must have delivery enabled.
+        # Set anne and bob's delivery status disabled by user.
+        self._pckdict['delivery_status'] = {
+            'anne@example.com': (2, 1612366744.399534),
+            'bob@example.com': (2, 1612366744.399534)}
+        import_config_pck(self._mlist, self._pckdict)
+        self.assertEqual(
+            self._mlist.owners.get_member('anne@example.com').delivery_status,
+            DeliveryStatus.enabled)
+        self.assertEqual(
+            self._mlist.moderators.get_member('bob@example.com').
+            delivery_status, DeliveryStatus.enabled)
+
     def test_owner_and_moderator_not_lowercase(self):
         # In the v2.1 pickled dict, the owner and moderator lists are not
         # necessarily lowercased already.
@@ -1139,7 +1318,7 @@ class TestRosterImport(unittest.TestCase):
     def test_nonmembers(self):
         import_config_pck(self._mlist, self._pckdict)
         expected = {
-            'gene': Action.accept,
+            'gene': Action.defer,
             'homer': Action.hold,
             'iris': Action.reject,
             'kenny': Action.discard,
@@ -1151,6 +1330,9 @@ class TestRosterImport(unittest.TestCase):
             member = self._mlist.nonmembers.get_member(
                 '{}@example.com'.format(name))
             self.assertEqual(member.moderation_action, action)
+            # Action.defer maps from accept; map it back to get the name.
+            if action == Action.defer:
+                action = Action.accept
             # Only regexps should remain in the list property.
             list_prop = getattr(
                 self._mlist,
@@ -1158,6 +1340,60 @@ class TestRosterImport(unittest.TestCase):
             self.assertEqual(len(list_prop), 1)
             self.assertTrue(all(addr.startswith('^') for addr in list_prop))
 
+    def test_nonmember_following_member(self):
+        self._pckdict['hold_these_nonmembers'] = [
+            'linda@example.com',
+            'homer@example.com',
+            ]
+        self._pckdict['members']['linda@example.com'] = 0
+        self._pckdict['user_options'] = {'linda@example.com': 1}
+        import_config_pck(self._mlist, self._pckdict)
+        member = self._mlist.nonmembers.get_member('linda@example.com')
+        self.assertEqual(member.moderation_action, Action.defer)
+        member = self._mlist.nonmembers.get_member('homer@example.com')
+        self.assertEqual(member.moderation_action, Action.hold)
+
+    def test_no_import_banned_address(self):
+        # Banned addresses should not be imported with any role.
+        self._pckdict['ban_list'] = [b'^.*example.com']
+        import_config_pck(self._mlist, self._pckdict)
+        self.assertEqual([], list(self._mlist.owners.addresses))
+        self.assertEqual([], list(self._mlist.moderators.addresses))
+        self.assertEqual([], list(self._mlist.members.addresses))
+        self.assertEqual([], list(self._mlist.nonmembers.addresses))
+
+
+class TestRosterVisibilityImport(unittest.TestCase):
+    """Test that member_roster_visibility is imported correctly.
+
+    Mailman 2.1 lists have a private_roster attribute to control roster
+    visibility with values 0==public, 1==members, 2==admins
+    These correspond to the Mailman 3 member_roster_visibility values
+    RosterVisibility.public, RosterVisibility.members and
+    RosterVisibility.moderators
+    """
+    layer = ConfigLayer
+
+    def setUp(self):
+        self._mlist = create_list('blank@example.com')
+        self._mlist.member_roster_visibility = DummyEnum.val
+
+    def _do_test(self, original, expected):
+        import_config_pck(self._mlist, dict(private_roster=original))
+        self.assertEqual(self._mlist.member_roster_visibility, expected)
+
+    def test_roster_visibility_public(self):
+        self._do_test(0, RosterVisibility.public)
+
+    def test_roster_visibility_members(self):
+        self._do_test(1, RosterVisibility.members)
+
+    def test_roster_visibility_moderators(self):
+        self._do_test(2, RosterVisibility.moderators)
+
+    def test_roster_visibility_bad(self):
+        self._do_test(3, DummyEnum.val)
+
 
 class TestPreferencesImport(unittest.TestCase):
     """Preferences get imported too."""
